@@ -556,21 +556,6 @@ def parse_file_resource(uri: str, root: Root) -> URL:
     )
 
 
-def parse_blob_resource(uri: str, root: Root) -> URL:
-    return uri_from_cli(
-        uri, root.client.username, root.client.cluster_name, allowed_schemes=("blob",)
-    )
-
-
-def parse_blob_or_file_resource(uri: str, root: Root) -> URL:
-    return uri_from_cli(
-        uri,
-        root.client.username,
-        root.client.cluster_name,
-        allowed_schemes=("blob", "file"),
-    )
-
-
 def parse_secret_resource(uri: str, root: Root) -> URL:
     return uri_from_cli(
         uri,
